/* demonstrates a fast integer 'expo' function
 * input, output, and expo amount are all integers in range -1024 <= value <= 1024.
 * if you wish to use a different range, use bit-shifts to scale to that range, call the function and then bit-shift the result back to the range you want
 * 
 * ceptimus June 2019
 */

int expo(int input, int xpo) {
  if (xpo == 0 || input == 0) {
    return input; // no need to calculate when zero xpo is applied
  }
  // clip values to keep within allowed range
  if (input < -1024) {
    input = -1024;
  } else if (input > 1024) {
    input = 1024;
  }
  if (xpo < -1024) {
    xpo = -1024;
  } else if (xpo > 1024) {
    xpo = 1024;
  }
  int32_t cube, i; // intermediate values use 32-bits.  input values (16-bits) are cast to 32-bit for calculations.

  cube = (int32_t)input * (int32_t) input; // first calculate square
  cube >>= 10; // shift ten bits (divide by 1024) to normalize
  cube *= input; // now cube
  cube >>= 10; // re-normalize
  cube *= xpo; // now xpo * input^3
  cube >>= 10; // re-normalize

  i = (int32_t)input * (int32_t)xpo;
  i >>= 10;
  
  int32_t output = (int32_t)input - i + cube;
  
  return (int)output;
}

void setup() {
  Serial.begin(115200);
  Serial.print("Expo\tInput\tOutput\n");
  for (int xpo = 0; xpo < 1024; xpo += 102) { // roughly 10% steps
    for (int input = -1024; input <= 1024; input += 128) {
      int output = expo(input, xpo);
      Serial.print(xpo);
      Serial.print("\t");
      Serial.print(input);
      Serial.print("\t");
      Serial.print(output);
      Serial.print("\n"); 
    }
    Serial.print("\n");
  }
}

void loop() {
}
